@@ -3,7 +3,7 @@ package PDF::API2;
 use strict;
 no warnings qw[ deprecated recursion uninitialized ];
 
-our $VERSION = '2.031'; # VERSION
+our $VERSION = '2.039'; # VERSION
 
 use Carp;
 use Encode qw(:all);
@@ -24,6 +24,8 @@ use PDF::API2::Resource::Shading;
 
 use PDF::API2::NamedDestination;
 
+use Scalar::Util qw(weaken);
+
 our @FontDirs = ( (map { "$_/PDF/API2/fonts" } @INC),
                   qw[ /usr/share/fonts /usr/local/share/fonts c:/windows/fonts c:/winnt/fonts ] );
 
@@ -71,9 +73,9 @@ PDF::API2 - Facilitates the creation and modification of PDF files
 
 =item $pdf = PDF::API2->new(%options)
 
-Creates a new PDF object.  If you will be saving it as a file and
-already know the filename, you can give the '-file' option to minimize
-possible memory requirements later on.
+Creates a new PDF object.  If you will be saving it as a file and already know
+the filename, you can give the '-file' option to minimize possible memory
+requirements later on.
 
 B<Example:>
 
@@ -89,6 +91,9 @@ B<Example:>
     ...
     $pdf->save();
 
+To turn off automatic compression of PDF contents, include option C<-compress>
+with a false value.  This is generally only useful for debugging.
+
 =cut
 
 sub new {
@@ -98,21 +103,27 @@ sub new {
     bless $self, $class;
     $self->{'pdf'} = PDF::API2::Basic::PDF::File->new();
 
-    $self->{'pdf'}->{' version'} = 4;
+    $self->{'pdf'}->{' version'} = '1.4';
     $self->{'pages'} = PDF::API2::Basic::PDF::Pages->new($self->{'pdf'});
     $self->{'pages'}->proc_set(qw(PDF Text ImageB ImageC ImageI));
     $self->{'pages'}->{'Resources'} ||= PDFDict();
     $self->{'pdf'}->new_obj($self->{'pages'}->{'Resources'}) unless $self->{'pages'}->{'Resources'}->is_obj($self->{'pdf'});
     $self->{'catalog'} = $self->{'pdf'}->{'Root'};
+    weaken $self->{'catalog'};
     $self->{'fonts'} = {};
     $self->{'pagestack'} = [];
-    $self->{'forcecompress'} = 1;
+    if (exists $options{'-compress'}) {
+        $self->{'forcecompress'} = $options{'-compress'} ? 1 : 0;
+    }
+    else {
+        $self->{'forcecompress'} = 1;
+    }
     $self->preferences(%options);
     if ($options{'-file'}) {
-        $self->{' filed'} = $options{'-file'};
         $self->{'pdf'}->create_file($options{'-file'});
+        $self->{'partial_save'} = 1;
     }
-    $self->{'infoMeta'}=[qw(Author CreationDate ModDate Creator Producer Title Subject Keywords)];
+    $self->{'infoMeta'} = [qw(Author CreationDate ModDate Creator Producer Title Subject Keywords)];
 
     my $version = eval { $PDF::API2::VERSION } || '(Unreleased Version)';
     $self->info('Producer' => "PDF::API2 $version [$^O]");
@@ -120,7 +131,7 @@ sub new {
     return $self;
 }
 
-=item $pdf = PDF::API2->open($pdf_file)
+=item $pdf = PDF::API2->open($pdf_file, %options)
 
 Opens an existing PDF file.
 
@@ -134,6 +145,9 @@ B<Example:>
     ...
     $pdf->update();
 
+To turn off automatic compression of PDF contents, include option C<-compress>
+with a false value.  This is generally only useful for debugging.
+
 =cut
 
 sub open {
@@ -141,30 +155,48 @@ sub open {
     croak "File '$file' does not exist" unless -f $file;
     croak "File '$file' is not readable" unless -r $file;
 
-    my $content;
-    my $scalar_fh = FileHandle->new();
-    CORE::open($scalar_fh, '+<', \$content) or die "Can't begin scalar IO";
-    binmode $scalar_fh, ':raw';
-
-    my $disk_fh = FileHandle->new();
-    CORE::open($disk_fh, '<', $file) or die "Can't open $file for reading: $!";
-    binmode $disk_fh, ':raw';
-    $disk_fh->seek(0, 0);
-    my $data;
-    while (not $disk_fh->eof()) {
-        $disk_fh->read($data, 512);
-        $scalar_fh->print($data);
+    my $self = {};
+    bless $self, $class;
+    foreach my $parameter (keys %options) {
+        $self->default($parameter, $options{$parameter});
     }
-    $disk_fh->close();
-    $scalar_fh->seek(0, 0);
 
-    my $self = $class->open_scalar($content, %options);
+    my $is_writable = -w $file;
+    $self->{'pdf'} = PDF::API2::Basic::PDF::File->open($file, $is_writable);
+    _open_common($self, %options);
     $self->{'pdf'}->{' fname'} = $file;
+    $self->{'opened_readonly'} = 1 unless $is_writable;
 
     return $self;
 }
 
-=item $pdf = PDF::API2->open_scalar($pdf_string)
+sub _open_common {
+    my ($self, %options) = @_;
+
+    $self->{'pdf'}->{'Root'}->realise();
+    $self->{'pdf'}->{' version'} ||= '1.3';
+
+    $self->{'pages'} = $self->{'pdf'}->{'Root'}->{'Pages'}->realise();
+    weaken $self->{'pages'};
+    my @pages = proc_pages($self->{'pdf'}, $self->{'pages'});
+    $self->{'pagestack'} = [sort { $a->{' pnum'} <=> $b->{' pnum'} } @pages];
+    weaken $self->{'pagestack'}->[$_] for (0 .. scalar @{$self->{'pagestack'}});
+
+    $self->{'catalog'} = $self->{'pdf'}->{'Root'};
+    weaken $self->{'catalog'};
+
+    if (exists $options{'-compress'}) {
+        $self->{'forcecompress'} = $options{'-compress'} ? 1 : 0;
+    }
+    else {
+        $self->{'forcecompress'} = 1;
+    }
+    $self->{'fonts'} = {};
+    $self->{'infoMeta'} = [qw(Author CreationDate ModDate Creator Producer Title Subject Keywords)];
+    return $self;
+}
+
+=item $pdf = PDF::API2->open_scalar($pdf_string, %options)
 
 Opens a PDF contained in a string.
 
@@ -179,6 +211,9 @@ B<Example:>
     ...
     $pdf->saveas('our/new.pdf');
 
+To turn off automatic compression of PDF contents, include option C<-compress>
+with a false value.  This is generally only useful for debugging.
+
 =cut
 
 # Deprecated (renamed)
@@ -198,16 +233,8 @@ sub open_scalar {
     CORE::open($fh, '+<', \$content) or die "Can't begin scalar IO";
 
     $self->{'pdf'} = PDF::API2::Basic::PDF::File->open($fh, 1);
-    $self->{'pdf'}->{'Root'}->realise();
-    $self->{'pages'} = $self->{'pdf'}->{'Root'}->{'Pages'}->realise();
-    $self->{'pdf'}->{' version'} ||= 3;
-    my @pages = proc_pages($self->{'pdf'}, $self->{'pages'});
-    $self->{'pagestack'} = [sort { $a->{' pnum'} <=> $b->{' pnum'} } @pages];
-    $self->{'catalog'} = $self->{'pdf'}->{'Root'};
-    $self->{'reopened'} = 1;
-    $self->{'forcecompress'} = 1;
-    $self->{'fonts'} = {};
-    $self->{'infoMeta'} = [qw(Author CreationDate ModDate Creator Producer Title Subject Keywords)];
+    _open_common($self, %options);
+    $self->{'opened_scalar'} = 1;
 
     return $self;
 }
@@ -576,10 +603,10 @@ sub version {
     my $self = shift();
     if (scalar @_) {
         my $version = shift();
-        croak "Invalid version $version" unless $version =~ /^(?:1\.)?([0-9]+)$/;
+        croak "Invalid version $version" unless $version =~ /^([12]\.[0-9]+)$/;
         $self->{'pdf'}->{' version'} = $1;
     }
-    return '1.' . $self->{'pdf'}->{' version'};
+    return $self->{'pdf'}->{' version'};
 }
 
 =item $bool = $pdf->isEncrypted()
@@ -629,15 +656,7 @@ sub info {
     if (scalar @_) {
         foreach my $k (@{$self->{'infoMeta'}}) {
             next unless defined $opt{$k};
-            if (is_utf8($opt{$k})) {
-                $self->{'pdf'}->{'Info'}->{$k} = PDFUtf($opt{$k} || 'NONE');
-            }
-            #elsif (is_utf8($opt{$k}) || utf8::valid($opt{$k})) {
-            #    $self->{'pdf'}->{'Info'}->{$k} = PDFUtf($opt{$k} || 'NONE');
-            #}
-            else {
-                $self->{'pdf'}->{'Info'}->{$k} = PDFStr($opt{$k} || 'NONE');
-            }
+            $self->{'pdf'}->{'Info'}->{$k} = PDFStr($opt{$k} || 'NONE');
         }
         $self->{'pdf'}->out_obj($self->{'pdf'}->{'Info'});
     }
@@ -841,10 +860,15 @@ sub pageLabel {
         $nums->add_elements(PDFNum($index));
 
         my $d = PDFDict();
-        $d->{'S'} = PDFName($opts->{'-style'} eq 'Roman' ? 'R' :
-                            $opts->{'-style'} eq 'roman' ? 'r' :
-                            $opts->{'-style'} eq 'Alpha' ? 'A' :
-                            $opts->{'-style'} eq 'alpha' ? 'a' : 'D');
+        if (defined $opts->{'-style'}) {
+            $d->{'S'} = PDFName($opts->{'-style'} eq 'Roman' ? 'R' :
+                                $opts->{'-style'} eq 'roman' ? 'r' :
+                                $opts->{'-style'} eq 'Alpha' ? 'A' :
+                                $opts->{'-style'} eq 'alpha' ? 'a' : 'D');
+        }
+        else {
+            $d->{'S'} = PDFName('D');
+        }
 
         if (defined $opts->{'-prefix'}) {
             $d->{'P'} = PDFStr($opts->{'-prefix'});
@@ -856,6 +880,8 @@ sub pageLabel {
 
         $nums->add_elements($d);
     }
+
+    return;
 }
 
 =item $pdf->finishobjects(@objects)
@@ -875,15 +901,17 @@ B<Example:>
 sub finishobjects {
     my ($self, @objs) = @_;
 
-    if ($self->{'reopened'}) {
+    if ($self->{'opened_scalar'}) {
         die "invalid method invocation: no file, use 'saveas' instead.";
     }
-    elsif ($self->{' filed'}) {
+    elsif ($self->{'partial_save'}) {
         $self->{'pdf'}->ship_out(@objs);
     }
     else {
         die "invalid method invocation: no file, use 'saveas' instead.";
     }
+
+    return;
 }
 
 sub proc_pages {
@@ -897,7 +925,7 @@ sub proc_pages {
 
     my @pages;
     $pdf->{' apipagecount'} ||= 0;
-    foreach my $page ($object->{'Kids'}->elementsof()) {
+    foreach my $page ($object->{'Kids'}->elements()) {
         $page->realise();
         if ($page->{'Type'}->val() eq 'Pages') {
             push @pages, proc_pages($pdf, $page);
@@ -931,7 +959,9 @@ B<Example:>
 
 sub update {
     my $self = shift();
-    $self->saveas($self->{'pdf'}->{' fname'});
+    croak "File is read-only" if $self->{'opened_readonly'};
+    $self->{'pdf'}->close_file();
+    return;
 }
 
 =item $pdf->saveas($file)
@@ -949,7 +979,7 @@ B<Example:>
 sub saveas {
     my ($self, $file) = @_;
 
-    if ($self->{'reopened'}) {
+    if ($self->{'opened_scalar'}) {
         $self->{'pdf'}->append_file();
         my $fh;
         CORE::open($fh, '>', $file) or die "Unable to open $file for writing: $!";
@@ -957,23 +987,30 @@ sub saveas {
         print $fh ${$self->{'content_ref'}};
         CORE::close($fh);
     }
-    elsif ($self->{' filed'}) {
+    elsif ($self->{'partial_save'}) {
         $self->{'pdf'}->close_file();
     }
     else {
-        $self->{'pdf'}->out_file($file);
+        unless ($self->{'pdf'}->{' fname'}) {
+            $self->{'pdf'}->out_file($file);
+        }
+        else {
+            $self->{'pdf'}->clone_file($file);
+            $self->{'pdf'}->close_file();
+        }
     }
 
     $self->end();
+    return;
 }
 
 sub save {
     my ($self, $file) = @_;
 
-    if ($self->{'reopened'}) {
+    if ($self->{'opened_scalar'}) {
         die "invalid method invocation: use 'saveas' instead.";
     }
-    elsif ($self->{' filed'}) {
+    elsif ($self->{'partial_save'}) {
         $self->{'pdf'}->close_file();
     }
     else {
@@ -981,6 +1018,7 @@ sub save {
     }
 
     $self->end();
+    return;
 }
 
 =item $string = $pdf->stringify()
@@ -1008,8 +1046,8 @@ B<Example:>
 sub stringify {
     my $self = shift();
 
-    my $str;
-    if ((defined $self->{'reopened'}) and ($self->{'reopened'} == 1)) {
+    my $str = '';
+    if ($self->{'opened_scalar'}) {
         $self->{'pdf'}->append_file();
         $str = ${$self->{'content_ref'}};
     }
@@ -1095,16 +1133,21 @@ sub page {
     }
     $page->{' apipdf'} = $self->{'pdf'};
     $page->{' api'} = $self;
+    weaken $page->{' apipdf'};
+    weaken $page->{' api'};
     $self->{'pdf'}->out_obj($page);
     $self->{'pdf'}->out_obj($self->{'pages'});
     if ($index == 0) {
         push @{$self->{'pagestack'}}, $page;
+        weaken $self->{'pagestack'}->[-1];
     }
     elsif ($index < 0) {
         splice @{$self->{'pagestack'}}, $index, 0, $page;
+        weaken $self->{'pagestack'}->[$index];
     }
     else {
         splice @{$self->{'pagestack'}}, $index - 1, 0, $page;
+        weaken $self->{'pagestack'}->[$index - 1];
     }
     # $page->{'Resources'} = $self->{'pages'}->{'Resources'};
     return $page;
@@ -1147,6 +1190,8 @@ sub openpage {
         bless $page, 'PDF::API2::Page';
         $page->{' apipdf'} = $self->{'pdf'};
         $page->{' api'} = $self;
+        weaken $page->{' apipdf'};
+        weaken $page->{' api'};
         $self->{'pdf'}->out_obj($page);
         if (($rotate = $page->find_prop('Rotate')) and (not defined($page->{' fixed'}) or $page->{' fixed'} < 1)) {
             $rotate = ($rotate->val() + 360) % 360;
@@ -1155,7 +1200,7 @@ sub openpage {
                 $page->{'Rotate'} = PDFNum(0);
                 foreach my $mediatype (qw(MediaBox CropBox BleedBox TrimBox ArtBox)) {
                     if ($media = $page->find_prop($mediatype)) {
-                        $media = [ map { $_->val() } $media->elementsof() ];
+                        $media = [ map { $_->val() } $media->elements() ];
                     }
                     else {
                         $media = [0, 0, 612, 792];
@@ -1192,7 +1237,7 @@ sub openpage {
             if ($self->default('pageencaps')) {
                 $content->{' stream'} .= ' q ';
             }
-            foreach my $k ($uncontent->elementsof()) {
+            foreach my $k ($uncontent->elements()) {
                 $k->realise();
                 $content->{' stream'} .= ' ' . unfilter($k->{'Filter'}, $k->{' stream'}) . ' ';
             }
@@ -1204,7 +1249,7 @@ sub openpage {
             # this will be fixed by the following code or content or filters
 
             ## if we like compress we will do it now to do quicker saves
-            if ($self->{'forcecompress'} > 0) {
+            if ($self->{'forcecompress'}) {
                 # $content->compressFlate();
                 $content->{' stream'} = dofilter($content->{'Filter'}, $content->{' stream'});
                 $content->{' nofilt'} = 1;
@@ -1219,7 +1264,8 @@ sub openpage {
     $self->{'pdf'}->out_obj($self->{'pages'});
     $page->{' apipdf'} = $self->{'pdf'};
     $page->{' api'} = $self;
-    $page->{' reopened'} = 1;
+    weaken $page->{' apipdf'};
+    weaken $page->{' api'};
     return $page;
 }
 
@@ -1243,7 +1289,7 @@ sub walk_obj {
 
     if (ref($source_object) =~ /Array$/) {
         $target_object->{' val'} = [];
-        foreach my $k ($source_object->elementsof()) {
+        foreach my $k ($source_object->elements()) {
             $k->realise() if ref($k) =~ /Objind$/;
             $target_object->add_elements(walk_obj($object_cache, $source_pdf, $target_pdf, $k));
         }
@@ -1329,7 +1375,7 @@ sub importPageIntoForm {
         # my $box = walk_obj($self->{'apiimportcache'}->{$s_pdf}, $s_pdf->{'pdf'}, $self->{'pdf'}, $s_page->{$k});
         next unless defined $s_page->find_prop($k);
         my $box = walk_obj($self->{'apiimportcache'}->{$s_pdf}, $s_pdf->{'pdf'}, $self->{'pdf'}, $s_page->find_prop($k));
-        $xo->bbox(map { $_->val() } $box->elementsof());
+        $xo->bbox(map { $_->val() } $box->elements());
         last;
     }
     $xo->bbox(0, 0, 612, 792) unless defined $xo->{'BBox'};
@@ -1363,7 +1409,7 @@ sub importPageIntoForm {
 
         $xo->{' stream'} = '';
         # openpage pages only contain one stream
-        my ($k) = $s_page->{'Contents'}->elementsof();
+        my ($k) = $s_page->{'Contents'}->elements();
         $k->realise();
         if ($k->{' nofilt'}) {
           # we have a finished stream here
@@ -1375,7 +1421,7 @@ sub importPageIntoForm {
             # so we just copy it and add the required "qQ"
             $xo->add('q', $k->{' stream'}, 'Q');
         }
-        $xo->compressFlate() if $self->{'forcecompress'} > 0;
+        $xo->compressFlate() if $self->{'forcecompress'};
     }
 
     return $xo;
@@ -1455,7 +1501,7 @@ sub import_page {
         my $box = walk_obj({}, $s_pdf->{'pdf'}, $self->{'pdf'}, $prop);
         my $method = lc $k;
 
-        $t_page->$method(map { $_->val() } $box->elementsof());
+        $t_page->$method(map { $_->val() } $box->elements());
     }
 
     $t_page->gfx->formimage($xo, 0, 0, 1);
@@ -1471,7 +1517,7 @@ sub import_page {
         }
         my @Fields = ();
         my @Annots = ();
-        foreach my $a ($s_page->{'Annots'}->elementsof()) {
+        foreach my $a ($s_page->{'Annots'}->elements()) {
             $a->realise();
             my $t_a = PDFDict();
             $self->{'pdf'}->new_obj($t_a);
@@ -1544,13 +1590,15 @@ sub pages {
     return scalar @{$self->{'pagestack'}};
 }
 
+=item ($llx, $lly, $urx, $ury) = $pdf->mediabox()
+
 =item $pdf->mediabox($name)
 
 =item $pdf->mediabox($w, $h)
 
 =item $pdf->mediabox($llx, $lly, $urx, $ury)
 
-Sets the global mediabox.
+Get or set the global mediabox.
 
 B<Example:>
 
@@ -1571,74 +1619,106 @@ B<Example:>
 
 =cut
 
-sub mediabox {
-    my ($self, $x1, $y1, $x2, $y2) = @_;
-    $self->{'pages'}->{'MediaBox'} = PDFArray(map { PDFNum(float($_)) } page_size($x1, $y1, $x2, $y2));
+sub _bounding_box {
+    my $self = shift();
+    my $type = shift();
+
+    # Get
+    unless (scalar @_) {
+        unless ($self->{'pages'}->{$type}) {
+            return if $type eq 'MediaBox';
+
+            # Use defaults per PDF 1.7 section 14.11.2 Page Boundaries
+            return $self->_bounding_box('MediaBox') if $type eq 'CropBox';
+            return $self->_bounding_box('CropBox');
+        }
+        return map { $_->val() } $self->{'pages'}->{$type}->elements();
+    }
+
+    # Set
+    $self->{'pages'}->{$type} = PDFArray(map { PDFNum(float($_)) } page_size(@_));
     return $self;
 }
 
+sub mediabox {
+    my $self = shift();
+    return $self->_bounding_box('MediaBox', @_);
+}
+
+=item ($llx, $lly, $urx, $ury) = $pdf->cropbox()
+
 =item $pdf->cropbox($name)
 
 =item $pdf->cropbox($w, $h)
 
 =item $pdf->cropbox($llx, $lly, $urx, $ury)
 
-Sets the global cropbox.
+Get or set the global cropbox.
+
+The cropbox defaults to the mediabox.
 
 =cut
 
 sub cropbox {
-    my ($self, $x1, $y1, $x2, $y2) = @_;
-    $self->{'pages'}->{'CropBox'} = PDFArray(map { PDFNum(float($_)) } page_size($x1, $y1, $x2, $y2));
-    return $self;
+    my $self = shift();
+    return $self->_bounding_box('CropBox', @_);
 }
 
+=item ($llx, $lly, $urx, $ury) = $pdf->bleedbox()
+
 =item $pdf->bleedbox($name)
 
 =item $pdf->bleedbox($w, $h)
 
 =item $pdf->bleedbox($llx, $lly, $urx, $ury)
 
-Sets the global bleedbox.
+Get or set the global bleedbox.
+
+The bleedbox defaults to the cropbox.
 
 =cut
 
 sub bleedbox {
-    my ($self, $x1, $y1, $x2, $y2) = @_;
-    $self->{'pages'}->{'BleedBox'} = PDFArray(map { PDFNum(float($_)) } page_size($x1, $y1, $x2, $y2));
-    return $self;
+    my $self = shift();
+    return $self->_bounding_box('BleedBox', @_);
 }
 
+=item ($llx, $lly, $urx, $ury) = $pdf->trimbox()
+
 =item $pdf->trimbox($name)
 
 =item $pdf->trimbox($w, $h)
 
 =item $pdf->trimbox($llx, $lly, $urx, $ury)
 
-Sets the global trimbox.
+Get or set the global trimbox.
+
+The trimbox defaults to the cropbox.
 
 =cut
 
 sub trimbox {
-    my ($self, $x1, $y1, $x2, $y2) = @_;
-    $self->{'pages'}->{'TrimBox'} = PDFArray(map { PDFNum(float($_)) } page_size($x1, $y1, $x2, $y2));
-    return $self;
+    my $self = shift();
+    return $self->_bounding_box('TrimBox', @_);
 }
 
+=item ($llx, $lly, $urx, $ury) = $pdf->artbox()
+
 =item $pdf->artbox($name)
 
 =item $pdf->artbox($w, $h)
 
 =item $pdf->artbox($llx, $lly, $urx, $ury)
 
-Sets the global artbox.
+Get or sets the global artbox.
+
+The artbox defaults to the cropbox.
 
 =cut
 
 sub artbox {
-    my ($self, $x1, $y1, $x2, $y2) = @_;
-    $self->{'pages'}->{'ArtBox'} = PDFArray(map { PDFNum(float($_)) } page_size($x1, $y1, $x2, $y2));
-    return $self;
+    my $self = shift();
+    return $self->_bounding_box('ArtBox', @_);
 }
 
 =back
@@ -1703,7 +1783,7 @@ sub corefont {
     require PDF::API2::Resource::Font::CoreFont;
     my $obj = PDF::API2::Resource::Font::CoreFont->new($self->{'pdf'}, $name, %opts);
     $self->{'pdf'}->out_obj($self->{'pages'});
-    $obj->tounicodemap() if $opts{-unicodemap} == 1;
+    $obj->tounicodemap() if $opts{-unicodemap};
     return $obj;
 }
 
@@ -1753,7 +1833,7 @@ sub psfont {
     my $obj = PDF::API2::Resource::Font::Postscript->new($self->{'pdf'}, $psf, %opts);
 
     $self->{'pdf'}->out_obj($self->{'pages'});
-    $obj->tounicodemap() if $opts{-unicodemap} == 1;
+    $obj->tounicodemap() if $opts{-unicodemap};
 
     return $obj;
 }
@@ -1840,7 +1920,7 @@ sub cjkfont {
     my $obj = PDF::API2::Resource::CIDFont::CJKFont->new($self->{'pdf'}, $name, %opts);
 
     $self->{'pdf'}->out_obj($self->{'pages'});
-    $obj->tounicodemap() if $opts{-unicodemap} == 1;
+    $obj->tounicodemap() if $opts{-unicodemap};
 
     return $obj;
 }
@@ -1883,13 +1963,19 @@ See Also: L<PDF::API2::Resource::Font::SynFont>
 =cut
 
 sub synfont {
-    my ($self, %opts) = @_;
+    my ($self, $font, %opts) = @_;
+
+    # PDF::API2 doesn't set BaseEncoding for TrueType fonts, so text
+    # isn't searchable unless a ToUnicode CMap is included.  Include
+    # the ToUnicode CMap by default, but allow it to be disabled (for
+    # performance and file size reasons) by setting -unicodemap to 0.
+    $opts{-unicodemap} = 1 unless exists $opts{-unicodemap};
 
     require PDF::API2::Resource::Font::SynFont;
-    my $obj = PDF::API2::Resource::Font::SynFont->new($self->{'pdf'}, %opts);
+    my $obj = PDF::API2::Resource::Font::SynFont->new($self->{'pdf'}, $font, %opts);
 
     $self->{'pdf'}->out_obj($self->{'pages'});
-    $obj->tounicodemap() if $opts{-unicodemap} == 1;
+    $obj->tounicodemap() if $opts{-unicodemap};
 
     return $obj;
 }
@@ -2004,6 +2090,11 @@ sub image_pnm {
 
 Imports and returns a new PNG image object.  C<$file> may be either a filename or a filehandle.
 
+Note: PNG files that include an alpha (transparency) channel go through a
+relatively slow process of separating the transparency channel into a PDF SMask
+object.  Install PDF::API2::XS or Image::PNG::Libpng to speed this up by an
+order of magnitude.
+
 =cut
 
 sub image_png {
@@ -2378,14 +2469,39 @@ __END__
 
 =back
 
+=head1 SUPPORTED PERL VERSIONS
+
+PDF::API2 will aim to support all major Perl versions that were released in the
+past six years, plus one, in order to continue working for the life of most
+long-term-stable (LTS) server distributions.
+
+For example, a version of PDF::API2 released on 2018-01-01 would support the
+last major version of Perl released before 2012-01-01, which happens to be 5.14.
+
+If you need to use this module on a server with an extremely out-of-date version
+of Perl, consider using either plenv or Perlbrew to run a newer version of Perl
+without needing admin privileges.
+
 =head1 KNOWN ISSUES
 
 This module does not work with perl's -l command-line switch.
 
 =head1 AUTHOR
 
-PDF::API2 was originally written by Alfred Reibenschuh.
+PDF::API2 was originally written by Alfred Reibenschuh, extending code written
+by Martin Hosken.
 
-It is currently being maintained by Steve Simms.
+It is currently being maintained and developed by Steve Simms.
+
+=head1 LICENSE
+
+This program is free software: you can redistribute it and/or modify it under
+the terms of the GNU Lesser General Public License as published by the Free
+Software Foundation, either version 2.1 of the License, or (at your option) any
+later version.
+
+This library is distributed in the hope that it will be useful, but WITHOUT ANY
+WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
+PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 
 =cut
